/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import java.util.HashMap;
import java.util.Map;

import org.conqat.ide.commons.ui.logging.LoggingUtils;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.command.BasicCommandStack;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.command.ChangeCommand;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.services.ICommandStackService;
import org.unicase.ecp.model.ECPWorkspaceManager;
import org.unicase.ecp.model.NoWorkspaceException;
import org.unicase.ecp.model.workSpaceModel.ECPProject;
import org.unicase.ecp.model.workSpaceModel.ECPWorkspace;
import org.unicase.ecp.model.workSpaceModel.WorkSpaceModelPackage;

/**
 * This class implements the {@link ICommandStackService} interface. We have to
 * mimic the command stack behavior, because EMFStore does not support redo of
 * commands. Once this capability is available, this service class could be
 * simplified.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @levd.rating RED Rev:
 */
public class CommandStackService implements ICommandStackService {

	/** Stores the workspace listener. */
	private AdapterImpl workspaceListener;

	/** Stores the EmfStore workspace. */
	private ECPWorkspace workspace;

	/** Stores the project command stacks. */
	private Map<ECPProject, BasicCommandStack> commandStackMap = new HashMap<ECPProject, BasicCommandStack>();

	/** Constructor. */
	public CommandStackService() {
		try {
			workspace = ECPWorkspaceManager.getInstance().getWorkSpace();
		} catch (NoWorkspaceException e) {
			LoggingUtils.log(ToolingKernelActivator.getDefault(),
					e.getMessage(), IStatus.ERROR, e);
			return;
		}

		createWorkspaceListener();
		workspace.eAdapters().add(workspaceListener);

		for (ECPProject project : workspace.getProjects()) {
			addCommandStack(project);
		}
	}

	/** {@inheritDoc} */
	@Override
	public void runAsCommand(EObject target, final Runnable runner) {
		ChangeCommand chgCommand = new ChangeCommand(target) {

			@Override
			protected void doExecute() {
				runner.run();
			}
		};
		commandStackMap.get(getECPProject(target)).execute(chgCommand);
	}

	/** {@inheritDoc} */
	@Override
	public boolean canUndo(EObject target) {
		return commandStackMap.get(getECPProject(target)).canUndo();
	}

	/** {@inheritDoc} */
	@Override
	public boolean canRedo(EObject target) {
		return commandStackMap.get(getECPProject(target)).canRedo();
	}

	/** {@inheritDoc} */
	@Override
	public void undo(EObject target) {
		commandStackMap.get(getECPProject(target)).undo();
	}

	/** {@inheritDoc} */
	@Override
	public void redo(EObject target) {
		commandStackMap.get(getECPProject(target)).redo();
	}

	/**
	 * Creates the workspace listener, which adds the command stacks for the
	 * added projects. It also unregisters command stacks from removed projects.
	 */
	private void createWorkspaceListener() {
		workspaceListener = new AdapterImpl() {

			@Override
			public void notifyChanged(Notification msg) {
				if ((msg.getFeatureID(ECPWorkspace.class)) == WorkSpaceModelPackage.ECP_WORKSPACE__PROJECTS) {
					if (msg.getEventType() == Notification.ADD
							&& WorkSpaceModelPackage.eINSTANCE.getECPProject()
									.isInstance(msg.getNewValue())) {
						addCommandStack((ECPProject) msg.getNewValue());
					} else if (msg.getEventType() == Notification.REMOVE
							&& WorkSpaceModelPackage.eINSTANCE.getECPProject()
									.isInstance(msg.getOldValue())) {
						removeCommandStack((ECPProject) msg.getOldValue());
					}
				}
				super.notifyChanged(msg);
			}
		};
	}

	/**
	 * Returns the model element's ECP project or <code>null</code> .
	 */
	private ECPProject getECPProject(EObject target) {
		try {
			return ECPWorkspaceManager.getInstance().getWorkSpace()
					.getProject(target);
		} catch (Exception e) {
			LoggingUtils.error(ToolingKernelActivator.getDefault(),
					"Unable to find ECP project!", e);
		}
		return null;
	}

	/** Adds the command stack for the given project. */
	private void addCommandStack(ECPProject project) {
		BasicCommandStack cmdStack = (BasicCommandStack) project
				.getEditingDomain().getCommandStack();
		commandStackMap.put(project, cmdStack);
	}

	/** Removes the command stack for the given project. */
	private void removeCommandStack(ECPProject project) {
		commandStackMap.remove(project);
	}
}
